@@ -103,11 +103,15 @@ pub struct Grammar
   pub Externtype : String,    // type of external structure
   pub Resynch : HashSet<String>, // resynchronization terminal symbols, ordered
   pub Errsym : String,        // error recovery terminal symbol
-  pub Lexnames : HashMap<String,String>, // print names of grammar symbols
+  pub Lexnames : Vec<(String,String)>, // print names of grammar symbols
   pub Extras : String,        // indicated by {% .. %}, mostly  use ...
   pub sametype: bool,  // determine if absyntype is only valuetype
   pub lifetime: String,
   pub tracelev:usize,
+  pub Lexvals: Vec<(String,String,String)>,  //"int" -> ("Num(n)","Val(n)")
+  pub Haslexval : HashSet<String>,
+  pub Lexextras: Vec<String>,
+  pub genlex: bool,
 }
 
 impl Default for Grammar {
@@ -132,11 +136,15 @@ impl Grammar
 //       Recover : HashSet::new(),
        Resynch : HashSet::new(),
        Errsym : String::new(),
-       Lexnames : HashMap::new(),
+       Lexnames : Vec::new(),
        Extras: String::new(),
        sametype:true,
        lifetime:String::new(), // empty means inferred
        tracelev:1,
+       Lexvals:Vec::new(),
+       Haslexval:HashSet::new(),
+       Lexextras:Vec::new(),
+       genlex: false,
      }
   }//new grammar
 
@@ -355,10 +363,34 @@ impl Grammar
                  self.Symbols[*index].precedence = preclevel;
                }
 	    }, // precedence and associativity
-	    "flexname" | "lexname"  => {
-               if stokens.len()<3 {continue;}
-               self.Lexnames.insert(stokens[1].to_string(),stokens[2].to_string());
+	    "lexname"  => {
+               if stokens.len()<3 {continue;}  // "||" -> "OROR"
+               self.Lexnames.push((stokens[2].to_string(),stokens[1].to_string()));
+	       self.Haslexval.insert(stokens[1].to_string());
+	       self.genlex = true;
             },
+	    "lexvalue" => {
+	       if stokens.len()<4 {continue;}  // "int" -> ("Num(n)","Val(n)")
+	       let mut valform = String::new();
+	       for i in 3 .. stokens.len()
+	       {
+	         valform.push_str(stokens[i]);
+		 if (i<stokens.len()-1) {valform.push(' ');}
+	       }
+	       self.Lexvals.push((stokens[1].to_string(),stokens[2].to_string(),valform));
+	       // record that this terminal always carries a value
+	       self.Haslexval.insert(stokens[1].to_string());
+	       self.genlex = true;
+	    },
+	    "lexset" | "lexattribute" => {
+	       let mut prop = String::new();
+	       for i in 1 .. stokens.len()
+	       {
+	          prop.push_str(stokens[i]); prop.push(' ');
+	       }
+	       self.Lexextras.push(prop);
+	       self.genlex = true;
+	    },
 //////////// case for grammar production:            
 	    LHS if (stokens[1]=="-->" || stokens[1]=="::=" || stokens[1]=="==>") => {
               if !foundeol && stokens[1]=="==>" {multiline=true; continue;}
@@ -611,7 +643,125 @@ impl Grammar
      }//while
      if nullable {Fseq.insert(la.to_owned());}
      Fseq
-  }//FirstSeq
+  }//FirstSeqb
+
+
+// procedure to generate lexical scanner from lexname, lexval and lexattribute
+// declarations in the grammar file.  Added for Version 0.2.3.  This procedure
+// is only used by other modules internally
+pub fn genlexer(&self,fd:&mut File, fraw:&str) -> Result<(),std::io::Error>
+{
+    ////// WRITE LEXER
+      let ref absyn = self.Absyntype;
+      let lifetime = if (self.lifetime.len()>0) {&self.lifetime} else {"'t"};
+      write!(fd,"\n// Lexical Scanner using RawToken and StrTokenizer\n")?;
+      let lexername = format!("{}lexer",&self.name);
+      let mut keywords:Vec<&str> = Vec::new();
+      let mut singles:Vec<char> = Vec::new();
+      let mut doubles:Vec<&str> = Vec::new();
+      // collect symbols from grammar
+      for symbol in &self.Symbols
+      {
+        if !symbol.terminal {continue;}
+        if is_alphanum(&symbol.sym) && &symbol.sym!="EOF" && &symbol.sym!="ANY_ERROR" && !self.Haslexval.contains(&symbol.sym) {
+	   keywords.push(&symbol.sym);
+	}
+	else if symbol.sym.len()==1 && !is_alphanum(&symbol.sym) {
+	   singles.push(symbol.sym.chars().next().unwrap());
+	}
+	else if symbol.sym.len()==2 && !is_alphanum(&symbol.sym) {
+	   doubles.push(&symbol.sym);
+	}
+      }//for each symbol
+      for (sym,_) in &self.Lexnames
+      {
+        if is_alphanum(sym) {continue;}
+	if sym.len()==1 {
+	   singles.push(sym.chars().next().unwrap());
+	}
+	else if sym.len()==2 {
+	   doubles.push(&sym);
+	}      
+      }// for symbols in lexnames such as "||" --> OROR
+      write!(fd,"pub struct {0}<'t> {{
+   stk: StrTokenizer<'t>,
+   keywords: HashSet<&'static str>,
+}}
+impl<'t> {0}<'t> 
+{{
+  pub fn from_str(s:&'t str) -> {0}<'t>  {{
+    Self::new(StrTokenizer::from_str(s))
+  }}
+  pub fn from_source(s:&'t LexSource<'t>) -> {0}<'t>  {{
+    Self::new(StrTokenizer::from_source(s))
+  }}
+  pub fn new(mut stk:StrTokenizer<'t>) -> {}<'t> {{
+    let mut keywords = HashSet::with_capacity(16);
+    for kw in [",&lexername)?; // end of write
+
+      for kw in &keywords {write!(fd,"\"{}\",",kw)?;}
+      write!(fd,"] {{keywords.insert(kw);}}
+    for c in [")?;
+      for c in singles {write!(fd,"'{}',",c)?;}
+      write!(fd,"] {{stk.add_single(c);}}
+    for d in [")?;
+      for d in doubles {write!(fd,"\"{}\",",d)?;}
+      write!(fd,"] {{stk.add_double(d);}}\n")?;
+    for attr in &self.Lexextras {write!(fd,"    stk.{};\n",attr.trim())?;}
+      write!(fd,"    {} {{stk,keywords}}\n  }}\n}}\n",&lexername)?;
+      // end of impl lexername
+      write!(fd,"impl<{0}> Tokenizer<{0},{1}> for {2}<{0}>
+{{
+   fn nextsym(&mut self) -> Option<TerminalToken<{0},{1}>> {{
+",lifetime,absyn,&lexername)?;
+      write!(fd,"    let tokopt = self.stk.next_token();
+    if let None = tokopt {{return None;}}
+    let token = tokopt.unwrap();
+    match token.0 {{
+")?;
+    if keywords.len()>0 {
+      write!(fd,"      RawToken::Alphanum(sym) if self.keywords.contains(sym) => Some(TerminalToken::{}(token,sym,<{}>::default())),\n",fraw,absyn)?;
+    }
+      // write special alphanums first - others might be "var" form
+      // next - write the Lexvals hexmap int -> (Num(n),Val(n))
+      for (tname,raw,val) in &self.Lexvals
+      {  
+        write!(fd,"      RawToken::{} => Some(TerminalToken::{}(token,\"{}\",{})),\n",raw,fraw,tname,val)?;
+      }
+      for (lform,tname) in &self.Lexnames
+      {
+        write!(fd,"      RawToken::Symbol(r\"{}\") => Some(TerminalToken::{}(token,\"{}\",<{}>::default())),\n",lform,fraw,tname,absyn)?;
+      }
+      write!(fd,"      RawToken::Symbol(s) => Some(TerminalToken::{}(token,s,<{}>::default())),\n",fraw,absyn)?;
+      write!(fd,"      RawToken::Alphanum(s) => Some(TerminalToken::{}(token,s,<{}>::default())),\n",fraw,absyn)?;      
+      write!(fd,"      _ => Some(TerminalToken::{}(token,\"<LexicalError>\",<{}>::default())),\n    }}\n  }}",fraw,absyn)?;
+      write!(fd,"
+   fn linenum(&self) -> usize {{self.stk.line()}}
+   fn column(&self) -> usize {{self.stk.column()}}
+   fn position(&self) -> usize {{self.stk.current_position()}}
+}}//impl Tokenizer
+\n")?;
+      Ok(())
+}//genlexer
+
 
 }//impl Grammar continued
 
+
+// used by genlexer routines
+fn is_alphanum(x:&str) -> bool
+{
+/*
+  let alphan = Regex::new(r"^[_a-zA-Z][_\da-zA-Z]*$").unwrap();
+  alphan.is_match(x)
+*/
+  if x.len()<1 {return false};
+  let mut chars = x.chars();
+  let first = chars.next().unwrap();
+  if !(first=='_' || first.is_alphabetic()) {return false;}
+  for c in chars
+  {
+    if !(c=='_' || c.is_alphanumeric()) {return false;}
+  }
+  true
+}//is_alphanum
